"use strict";(self.webpackChunkPeacasso=self.webpackChunkPeacasso||[]).push([[47],{2493:function(e,n,t){t.r(n);var a=t(7294),s=t(8782),c=t(7699);n.default=function(){return a.createElement(s.Z,{title:"Sign In",showHeader:!1},a.createElement(c.Z,null))}}}]);
//# sourceMappingURL=component---src-pages-login-tsx-21d00f12a94e98daf01d.js.map