"use strict";(self.webpackChunkPeacasso=self.webpackChunkPeacasso||[]).push([[196],{7926:function(e,t,n){n.r(t),n.d(t,{default:function(){return c}});var l=n(7294),r=n(8782),u=function(e){e.data;var t=l.useRef(null),n=l.useRef(null),r=null;return l.useEffect((function(){null!=t.current&&null!=n.current&&(r=t.current.getContext("2d"),t.current.width=n.current.clientWidth,t.current.height=n.current.clientHeight,{x:t.current.width/2,y:t.current.height/2},r.fillStyle="red",r.fillRect(0,0,100,100),setTimeout((function(){r.fillStyle="blue",r.fillRect(0,0,50,50),r.scale(.5,.5)}),2e3))}),[]),l.createElement("div",{style:{minHeight:"400px"},ref:n,className:"w-full  "},l.createElement("canvas",{ref:t,className:"bordper w-full editor",id:"canvas"}))},a=function(e){e.data;return l.createElement("div",null,"Editor",l.createElement(u,null))},c=function(e){var t=e.data;return l.createElement(r.Z,{meta:t.site.siteMetadata,title:"Home",link:"/"},l.createElement("main",{className:""},l.createElement(a,null)))}}}]);
//# sourceMappingURL=component---src-pages-editor-tsx-664cdf67b397ad56b6de.js.map