"use strict";(self.webpackChunkPeacasso=self.webpackChunkPeacasso||[]).push([[218],{2513:function(e,t,n){n.r(t);var a=n(7294),l=n(1597),o={color:"#232129",padding:"96px",fontFamily:"-apple-system, Roboto, sans-serif, serif"},r={marginTop:0,marginBottom:64,maxWidth:320},i={marginBottom:48};t.default=function(){return a.createElement("main",{style:o},a.createElement("title",null,"Not found"),a.createElement("h1",{style:r},"Page not found"),a.createElement("p",{style:i},"Sorry"," ",a.createElement("span",{role:"img","aria-label":"Pensive emoji"},"😔")," ","we couldn’t find what you were looking for.",a.createElement("br",null),null,a.createElement("br",null),a.createElement(l.Link,{to:"/"},"Go home"),"."))}}}]);
//# sourceMappingURL=component---src-pages-404-tsx-35860b35f750ed726c73.js.map